@@ -85,7 +85,6 @@ def registerType(type_, module_):
 	global __mlTypes
 	__mlTypes[type_] = module_
 
-# Duplicate key exception
 class DuplicateException(Exception):
 	"""DuplicateException class
 
@@ -93,6 +92,13 @@ class DuplicateException(Exception):
 	"""
 	pass
 
+class RecordException(Exception):
+	"""RecordException class
+
+	Used for raising issues with the underlying definition of the record
+	"""
+	pass
+
 class RevisionException(Exception):
 	"""Revision Exception
 
@@ -255,11 +261,11 @@ class Record(abc.ABC):
 			if sMD5 != sHash:
 
 				# Update the revision
-				self._dRecord[info['rev_field']] = '%d-%s' % (int(sVer)+1, sMD5)
+				self._dRecord[struct['rev_field']] = '%d-%s' % (int(sVer)+1, sMD5)
 
 				# Flag the revision field as changed
 				if self._dChanged != True:
-					self._dChanged[info['rev_field']] = True
+					self._dChanged[struct['rev_field']] = True
 
 				# Return OK
 				return True
@@ -412,7 +418,7 @@ class Record(abc.ABC):
 				'append' optional postfix for dynamic DBs
 
 		Returns:
-			mixed|None
+			uint
 		"""
 		raise NotImplementedError('Must implement the "createMany" method')
 
@@ -454,7 +460,8 @@ class Record(abc.ABC):
 	def exists(cls, _id, index=None, custom={}):
 		"""Exists
 
-		Returns true if the specified ID or unique index value exists
+		Returns the ID (primary key) of the record for the specified ID or
+		unique index value found, else False if no record is found
 
 		Arguments:
 			_id (mixed): The ID to check
@@ -464,7 +471,7 @@ class Record(abc.ABC):
 				'append' optional postfix for dynamic DBs
 
 		Returns:
-			bool
+			mixed|False
 		"""
 		raise NotImplementedError('Must implement the "exists" method')
 
