@@ -30,6 +30,7 @@ MAX_RETRIES = 3
 
 # Backwards compatibility (and ease of use)
 DuplicateException = Record_Base.DuplicateException
+RecordException = Record_Base.RecordException
 
 def _connect(host, error_count=0):
 	"""Connect
@@ -582,7 +583,7 @@ class Record(Record_Base.Record):
 				'append' optional postfix for dynamic DBs
 
 		Returns:
-			mixed[]
+			uint
 		"""
 
 		# Make sure conflict arg is valid
@@ -624,7 +625,7 @@ class Record(Record_Base.Record):
 
 				# If it's a duplicate key
 				if dRes['first_error'][0:21] == 'Duplicate primary key':
-					raise Record_Base.DuplicateException(self._dStruct['primary'], self._dRecord[self._dStruct['primary']])
+					raise Record_Base.DuplicateException(dStruct['primary'])
 
 				# Else, it's an unknown error
 				else:
@@ -637,7 +638,7 @@ class Record(Record_Base.Record):
 			# Go through each record and store the primary key if we didn't set
 			#	it ourselves
 			if dStruct['auto_primary']:
-				for i in xrange(len(records)):
+				for i in range(len(records)):
 					records[i][dStruct['primary']] = dRes['generated_keys'][i]
 
 				# Return the primary keys
@@ -852,7 +853,8 @@ class Record(Record_Base.Record):
 	def exists(cls, _id, index=None, custom={}):
 		"""Exists
 
-		Returns true if the specified primary key or unique index value exists
+		Returns the ID (primary key) of the record for the specified ID or
+		unique index value found, else False if no record is found
 
 		Arguments:
 			_id (mixed): The primary key to check
@@ -870,11 +872,12 @@ class Record(Record_Base.Record):
 
 		# Use the get method to avoid duplicate code and check if anything was
 		#	returned
-		if not cls.get(_id, index=index, raw=[dStruct['primary']], custom=custom):
+		dRecord = cls.get(_id, index=index, raw=[dStruct['primary']], custom=custom)
+		if not dRecord:
 			return False
 
-		# If anything was returned, the key exists
-		return True
+		# If anything was returned, return the primary key
+		return dRecord[dStruct['primary']]
 
 	# filter static method
 	@classmethod
