@@ -11,9 +11,9 @@ __email__ = "chris@ouroboroscoding.com"
 __created__ = "2018-11-11"
 
 # Python imports
+from datetime import datetime
 from hashlib import sha1
 from time import sleep, time
-from datetime import datetime
 
 # Pip imports
 import requests
@@ -41,10 +41,10 @@ __funcToRequest = {
 }
 """Map functions to REST types"""
 
-def __request(service, action, path, data, sesh=None, environ=None):
+def request(service, action, path, data, sesh=None, environ=None):
 	"""Request
 
-	Internal method to convert REST requests into HTTP requests
+	Method to convert REST requests into HTTP requests
 
 	Arguments:
 		service (str): The service we are requesting data from
@@ -68,13 +68,16 @@ def __request(service, action, path, data, sesh=None, environ=None):
 		if 'instance' in __mdRegistered[service]:
 
 			# If verbose requested
-			if __mbVerbose: print('%s: Calling %s.%s("%s", %s)' % (str(datetime.now()), service, action, path, str(data)))
+			if __mbVerbose: print('%s: Calling %s.%s("%s", %s)' % (str(datetime.now()), service, action, path, JSON.encode(data, 2)))
 
 			# Directly call the action
 			oResponse = getattr(__mdRegistered[service]['instance'], action)(
 				path, data, sesh, environ
 			)
 
+			# If verbose requested
+			if __mbVerbose:	print('%s: Returning %s\n' % (str(datetime.now()), JSON.encode(oResponse.toDict(), 2)))
+
 		# Else if the service is running elsewhere
 		else:
 
@@ -84,10 +87,6 @@ def __request(service, action, path, data, sesh=None, environ=None):
 			# Generate the URL to reach the service
 			sURL = __mdRegistered[service]['url'] + path
 
-			# If verbose requested
-			if __mbVerbose:
-				print('%s: Calling %s %s %s)' % (str(datetime.now()), __funcToRequest[action][1], sURL, str(data)))
-
 			# Convert the data to JSON
 			sData = JSON.encode(data)
 
@@ -142,9 +141,6 @@ def __request(service, action, path, data, sesh=None, environ=None):
 			# Else turn the content into an Response and return it
 			oResponse = Response.fromJSON(oRes.text)
 
-		# If verbose requested
-		if __mbVerbose:	print('%s: Returning %s\n' % (str(datetime.now()), str(oResponse)))
-
 		# Return the Response of the request
 		return oResponse
 
@@ -167,7 +163,7 @@ def create(service, path, data, sesh=None, environ=None):
 	Returns:
 		Response
 	"""
-	return __request(service, 'create', path, data, sesh, environ)
+	return request(service, 'create', path, data, sesh, environ)
 
 def delete(service, path, data, sesh=None, environ=None):
 	"""Delete
@@ -184,7 +180,7 @@ def delete(service, path, data, sesh=None, environ=None):
 	Returns:
 		Response
 	"""
-	return __request(service, 'delete', path, data, sesh, environ)
+	return request(service, 'delete', path, data, sesh, environ)
 
 def internalKey(key = None):
 	"""Internal Key
@@ -252,7 +248,7 @@ def read(service, path, data, sesh=None, environ=None):
 	Returns:
 		Response
 	"""
-	return __request(service, 'read', path, data, sesh, environ)
+	return request(service, 'read', path, data, sesh, environ)
 
 def register(services, restconf, salt, internal=5):
 	"""Register
@@ -332,7 +328,7 @@ def update(service, path, data, sesh=None, environ=None):
 	Returns:
 		Response
 	"""
-	return __request(service, 'update', path, data, sesh, environ)
+	return request(service, 'update', path, data, sesh, environ)
 
 def verbose(flag=True):
 	"""Verbose
@@ -413,7 +409,7 @@ class Response(object):
 				# Else, try to pull out the code and message
 				else:
 					self.error = {"code": error.args[0], "msg": ''}
-					if len(error.args) > 1: dErr['msg'] = error.args[1]
+					if len(error.args) > 1: self.error['msg'] = error.args[1]
 
 			# Else, we got something invalid
 			else:
@@ -628,6 +624,9 @@ class Service(object):
 	The object to build all Services from
 	"""
 
+	__pathToDef = {}
+	"""Map of paths to function name"""
+
 	def create(self, path, data, sesh=None, environ=None):
 		"""Create
 
@@ -732,6 +731,11 @@ class Service(object):
 		Returns:
 			Service
 		"""
+
+		# Init the paths to function names
+		self._pathToDef
+
+		# Return self for chaining
 		return self
 
 	@classmethod
@@ -845,8 +849,8 @@ class Service(object):
 		except ResponseException as e:
 			return e.args[0]
 
-	@staticmethod
-	def pathToMethod(path, append=''):
+	@classmethod
+	def pathToMethod(cls, path, append=''):
 		"""Path to Method
 
 		Takes a path and converts it to the standard naming for Service methods
@@ -858,14 +862,28 @@ class Service(object):
 		Returns:
 			str
 		"""
-		sRet = ''
-		iLen = len(path)
-		i = 0
-		while i < iLen:
-			if(path[i] in ['/', '_', '-']):
+
+		# If we already have the function name associated
+		try:
+			sRet = cls.__pathToDef[path]
+
+		# Else, if we haven't generated it yet
+		except KeyError:
+
+			# Generate the name from the path
+			sRet = ''
+			iLen = len(path)
+			i = 0
+			while i < iLen:
+				if(path[i] in ['/', '_', '-']):
+					i += 1
+					sRet += path[i].upper()
+				else:
+					sRet += path[i]
 				i += 1
-				sRet += path[i].upper()
-			else:
-				sRet += path[i]
-			i += 1
+
+			# Store it for next time
+			cls.__pathToDef[path] = sRet
+
+		# Return the generated function name plus the append
 		return sRet + append
